"""
Module contains standard command line arguments parser for manager
"""
from argparse import ArgumentParser
from dataclasses import asdict, dataclass
from typing import Optional, Set

from ..web.cmd_parser import AppCMDOptions, ApplicationCMDParser, notNegativeInt


@dataclass
class ManagerCMDOptions(AppCMDOptions):
    """
    Manager cmd options
    """

    # auto reload configuration
    configReload: bool
    # configuration pulling period
    configPullingPeriod: int


class ManagerCMDParser:  # pylint: disable-msg=too-few-public-methods
    """
    Command line parser
    Attributes:
        argParser (ArgumentParser): argument parser
        tagableSettings (Set[str]): set of setting names which allowed tags for pulling
        defaultConfigFilePath (str): default config path

    """

    def __init__(
        self,
        defaultConfigFilePath: Optional[str] = None,
        tagableSettings: Optional[Set[str]] = None,
        defaultPort: int | None = None,
    ):
        self.tagableSettings = tagableSettings
        self.defaultConfigFilePath = defaultConfigFilePath

        self.parentArgParser = ApplicationCMDParser(
            defaultPort=defaultPort, defaultConfigFilePath=defaultConfigFilePath, tagableSettings=tagableSettings
        )
        epilogAboutEnv = """
            Service settings can be load from config file or configurator.
            It is also possible to override each setting from environment variables, which will take precedence.
            To redefine setting use `VL_SETTINGS` prefix.
            Examples:
                - VL_SETTINGS.INFLUX_MONITORING.SEND_DATA_FOR_MONITORING=0
                  in this case `SEND_DATA_FOR_MONITORING` from `INFLUX_MONITORING` will be set to 0
                - VL_SETTINGS.OTHER.STORAGE_TIME=LOCAL
                  (for non composite settings require to specify `OTHER` section in the same way as in config file)
                  in this case `STORAGE_TIME` (if service use the setting) will be set to `LOCAL`
        """
        argParser = ArgumentParser(parents=[self.parentArgParser.argParser], epilog=epilogAboutEnv)

        argParser.add_argument(
            "--config-reload",
            "-cf",
            default=0,
            type=int,
            help="reload configuration in real time",
            const=True,
            nargs="?",
            choices=[0, 1],
        )
        argParser.add_argument(
            "--pulling-time",
            default=10,
            type=notNegativeInt,
            help="period of configuration pulling for configuration reloading, in seconds",
        )
        self.argParser = argParser

    def parse(self) -> ManagerCMDOptions:
        """
        Parse command line
        Returns:
            parsed options
        """
        args = self.argParser.parse_known_args()[0]  # pylint: disable-msg=W0612
        return ManagerCMDOptions(
            **asdict(self.parentArgParser.parse()),
            configReload=bool(args.config_reload),
            configPullingPeriod=args.pulling_time,
        )
