""" Module release connection checks for Luna services and databases """
import asyncio
import uuid
from numbers import Real
from pathlib import Path, PosixPath
from typing import Any, Awaitable, Callable, List, Optional, Set, Tuple, Union

from log import DEFAULT_REQUEST_ID
from luna3.common.exceptions import LunaApiException
from urllib3.exceptions import LocationValueError
from vlutils.helpers import syncCallable

from ..constants.paths import APP_ROOT
from ..errors.errors import Error
from .healthcheck import Health
from .log import Logger


def checkInflux(
    bucket: str,
    host: str,
    port: int = 8086,
    organization: str = None,
    token: str = None,
    ssl: bool = False,
    asyncCheck: bool = False,
) -> Union[Awaitable[Health], Health]:
    """
    Influx v.2 health check function.
    Args:
        host: influx host
        port: influx port
        organization: influx 2.x workspace
        token: influx 2.x auth token
        bucket: influx 2.x bucket
        ssl: use https or not
        asyncCheck: whether to async health check
    Returns:
        Health status
    """
    from influxdb_client import InfluxDBClient  # pylint: disable-msg=C0415
    from influxdb_client.rest import ApiException  # pylint: disable-msg=C0415

    checkerName = "Influx v.2"
    logger = Logger(f"{checkerName} health checker")

    scheme = "https" if ssl else "http"
    baseUrl = f"{scheme}://{host}:{port}"

    def check() -> Health:
        client = InfluxDBClient(url=baseUrl, token=token, org=organization, verify_ssl=ssl)
        try:
            reply = client.buckets_api().find_bucket_by_name(bucket)
        except (LocationValueError, ApiException) as exc:
            logger.error(f"Connection refused: {exc}")
            return Health(checkerName, False, Error.HealthCheckError.format(str(exc)))
        except Exception as exc:  # pylint: disable-msg=broad-except
            logger.error(f"Unknown error: {exc}")
            return Health(checkerName, False, Error.HealthCheckError.format(str(exc)))
        if reply:
            return Health(checkerName, True, Error.Success)
        errorMsg = f"Bucket {bucket} not found"
        logger.error(errorMsg)
        return Health(checkerName, False, Error.HealthCheckError.format(errorMsg))

    if asyncCheck:
        return asyncio.get_event_loop().run_in_executor(None, check)
    return check()


def _checkServiceConnection(
    logger: Logger,
    client: "luna3.common.base_api.BaseAPI",
    service: str,
    closeSession: Optional[bool] = None,
    asyncCheck: bool = False,
    totalTimeout: Optional[Real] = None,
    connectTimeout: Optional[Real] = None,
    sockReadTimeout: Optional[Real] = None,
    sockConnectTimeout: Optional[Real] = None,
) -> Union[bool, Awaitable[bool]]:
    """
    Check connection to any initialised BaseAPI.

    Args:
        logger: logger to log
        client: client to check
        service: service name
        closeSession: whether to close session (True - yes, False - no, None - default)
        asyncCheck: make synchronous or asynchronous check
        totalTimeout: total request timeout
        connectTimeout: socket connect timeout or connection wait from pool timeout
        sockReadTimeout: timeout for reading a chunk from socket
        sockConnectTimeout: socket connect timeout

    Returns:
        True, if check was passed successfully, else False.
    """

    async def check():
        logger.info("begin")
        result = await serviceCheck(
            checkerName=service,
            client=client,
            logger=logger,
            closeSession=closeSession,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockConnectTimeout=sockConnectTimeout,
            sockReadTimeout=sockReadTimeout,
        )
        if result.status is True:
            logger.info("success")
        return result.status

    return syncCallable(check)(asyncCall=asyncCheck)


def checkConnectionToLunaImageStore(
    origin: str, apiVersion: int, bucket: str, *, postfix: str = "", asyncCheck: bool = False, **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Image Store and bucket existence.

    Args:
        origin: Luna Image Store origin
        apiVersion: Luna Image Store api version
        bucket: Luna Image Store bucket
        postfix: specified postfix to use in logs
        asyncCheck: make synchronous or asynchronous check

    Returns:
        True, if check was passed successfully, else False.
    """

    async def check():
        from luna3.image_store.image_store import StoreApi  # pylint: disable-msg=C0415

        serviceName = "Luna Image Store"
        logger = Logger(template=f"Check connection to {serviceName} {postfix}")
        luna3Client: StoreApi = StoreApi(origin, apiVersion)
        success = await _checkServiceConnection(
            logger, luna3Client, serviceName, closeSession=False, asyncCheck=True, **kwargs
        )
        try:
            if success:
                # some client security service forbids to make a request to some S3 endpoints (/buckets or
                # /<bucket> for example)
                # therefore we try to get an image and analyze error
                response = await luna3Client.getImage(
                    asyncRequest=True,
                    lunaRequestId=DEFAULT_REQUEST_ID,
                    imageId=str(uuid.UUID(int=0)),
                    bucketName=bucket,
                    **kwargs,
                )
                if response.success:
                    # can get image => bucket exists
                    return True
                if response.statusCode == 404:
                    if response.json["error_code"] == 13005:
                        # bucket not found
                        logger.error(f"Bucket '{bucket}' not found")
                        return False
                    if response.json["error_code"] == 13003:
                        # image not found but bucket exists
                        return True
                # something went wrong
                logger.error(f"error occurred: {response.json}")
                return False
            return False
        finally:
            await luna3Client.close()

    return syncCallable(check)(asyncCall=asyncCheck)


def checkConnectionToLunaFaces(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Faces.

    Args:
        origin: Luna Faces origin
        apiVersion: Luna Faces api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.faces.faces import FacesApi  # pylint: disable-msg=C0415

    serviceName = "Luna Faces"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = FacesApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaRemoteSDK(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Remote SDK.

    Args:
        origin: Luna Remote SDK origin
        apiVersion: Luna Remote SDK api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.remote_sdk.remote_sdk import LunaRemoteSDKApi  # pylint: disable-msg=C0415

    serviceName = "Luna Remote SDK"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = LunaRemoteSDKApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaEvents(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Events.

    Args:
        origin: Luna Events origin
        apiVersion: Luna Events api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.events.events import EventsApi  # pylint: disable-msg=C0415

    serviceName = "Luna Events"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = EventsApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaTasks(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Tasks.

    Args:
        origin: Luna Tasks origin
        apiVersion: Luna Tasks api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.tasks.tasks import TasksApi  # pylint: disable-msg=C0415

    serviceName = "Luna Tasks"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = TasksApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaHandlers(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Handlers.

    Args:
        origin: Luna Handlers origin
        apiVersion: Luna Handlers api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.handlers.handlers import HandlersApi  # pylint: disable-msg=C0415

    serviceName = "Luna Handlers"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = HandlersApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaPythonMatcher(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Python Matcher.

    Args:
        origin: Luna Python Matcher origin
        apiVersion: Luna Python Matcher api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.python_matcher.python_matcher import PythonMatcherApi  # pylint: disable-msg=C0415

    serviceName = "Luna Python Matcher"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = PythonMatcherApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaAccounts(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Accounts.

    Args:
        origin: Luna Accounts origin
        apiVersion: Luna Accounts api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.accounts.accounts import AccountsApi  # pylint: disable-msg=C0415,E0611,E0401

    serviceName = "Luna Accounts"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = AccountsApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToDB(
    dbSetting: "DBSetting",
    modelNames: Optional[List[str]] = None,  # pylint: disable-msg=W0613
    checkMigrationVersion: bool = True,
    *,
    postfix: str = "",
    asyncCheck: bool = False,
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to DB. Connection, migration version and tables availability are checked.

    Args:
        dbSetting: database setting
        modelNames: model names list to check (table names and view names)
        checkMigrationVersion: check migration version
        postfix: specified postfix to use in logs
        asyncCheck: whether to async check connection

    Returns:
        True, if check was passed successfully, else False.
    """
    from sqlalchemy import select  # pylint: disable-msg=C0415

    from .adaptor_engine import UniAdaptorEngine  # pylint: disable-msg=C0415

    logger = Logger(template=f"Check connection to db {postfix}")
    logger.info("begin")

    async def check():
        try:
            async with UniAdaptorEngine(dbSetting=dbSetting).adaptorContext() as uniAdaptor:
                await uniAdaptor.execute(select([1]))
                if checkMigrationVersion:
                    currentRevision, expectedRevision = await uniAdaptor.inspectAlembicRevision()

                    if currentRevision != expectedRevision:
                        raise ConnectionError(
                            f"Actual and expected db revisions are not equal (actual '{currentRevision}', expected "
                            f"'{expectedRevision}'); please check that you have executed the database migration"
                        )

                tableNames = await uniAdaptor.inspectDBSchema()
                for modelName in modelNames or []:
                    if modelName not in tableNames:
                        raise ConnectionError(f'"{modelName}" model: FAIL')

                logger.info("success")
                return True
        except TypeError:
            # raised when function got wrong count of args or wrong kwargs
            raise
        except Exception as ex:  # pylint: disable-msg=broad-except
            logger.error(f"Failed check connection to database: {ex}")
            return False

    return syncCallable(check)(asyncCall=asyncCheck)


def checkConnectionToSentinel(
    sentinels: List[Tuple[str, int]],
    masterName: str,
    sentinelUser: str = "",
    sentinelPassword: str = "",
    redisUser: str = "",
    redisPassword: str = "",
    *,
    postfix: str = "",
    asyncCheck: bool = False,
) -> Union[Awaitable[bool], bool]:
    """
    Check connection to Redis Sentinel.

    Args:
        sentinels: A List of tuples (host, port), addresses of sentinels
        masterName: A string, name of the group of redis instances
        sentinelUser: sentinel username
        sentinelPassword: sentinel password
        redisUser: master username
        redisPassword: master password
        postfix: specified postfix to use in logs
        asyncCheck: whether to async check connection

    Returns:
        True, if check was passed successfully, else False.
    """
    from redis.asyncio import Sentinel  # pylint: disable-msg=C0415

    async def check():
        logger = Logger(template=f"Check connection to sentinel and redis {postfix}")
        logger.info("begin")
        try:
            sentinel = Sentinel(
                sentinels=sentinels,
                sentinel_kwargs={"username": sentinelUser, "password": sentinelPassword},
                username=redisUser,
                password=redisPassword,
            )
            async with sentinel.master_for(masterName) as client:
                await client.ping()
            logger.info("success")
            return True
        except Exception as ex:  # pylint: disable-msg=broad-except
            sentinelSettings = {"sentinels": sentinels, "master_name": masterName}
            logger.error(f"Failed check connection to sentinel: {ex}, with settings: {sentinelSettings}")
            return False

    return syncCallable(check)(asyncCall=asyncCheck)


def checkConnectionToRedis(
    redisHost: str,
    redisPort: int,
    redisUser: str = "",
    redisPassword: str = "",
    *,
    postfix: str = "",
    asyncCheck: bool = False,
) -> Union[Awaitable[bool], bool]:
    # pylint: disable-msg=W0613
    """
    Check connection to Redis.

    Args:
        redisHost: redis host
        redisPort: redis port
        redisUser: redis username
        redisPassword: redis password
        postfix: specified postfix to use in logs
        asyncCheck: whether to async check connection

    Returns:
        True, if check was passed successfully, else False.
    """
    from redis.asyncio import Redis  # pylint: disable-msg=C0415

    async def check():
        logger = Logger(template=f"Check connection to redis {postfix}")
        logger.info("begin")

        try:
            async with Redis(host=redisHost, port=redisPort, username=redisUser, password=redisPassword) as conn:
                await conn.ping()
            logger.info("success")
            return True
        except Exception as ex:  # pylint: disable-msg=broad-except
            redisSettings = {"host": redisHost, "port": redisPort}
            logger.error(f"Failed check connection to redis: {ex}, with settings: {redisSettings}")
            return False

    return syncCallable(check)(asyncCall=asyncCheck)


def checkConnectionToInflux(
    bucket: str,
    host: str,
    port: int = 8086,
    organization: str = None,
    token: str = None,
    ssl: bool = False,
    asyncCheck: bool = False,
) -> bool:
    """
    Function for check of connection to Influx 2.x for Luna-API monitoring. Connection and series existence.

    Args:
        host: influx host
        port: influx port
        organization: influx 2.x workspace
        token: influx 2.x auth token
        bucket: influx 2.x bucket
        ssl: use https or not
        asyncCheck: whether to async check connection

    Returns:
        True, if check was passed successfully, else False.
    """
    logger = Logger(template="Check connection to Influxdb")

    def check() -> bool:
        logger.info("begin")
        result = checkInflux(bucket, host, port, organization, token, ssl, asyncCheck)
        if result.status is True:
            logger.info("success")
        return result.status

    return check()


def checkConnectionToLunaSender(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Sender.

    Args:
        origin: Luna Sender origin
        apiVersion: Luna Sender api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.sender.wsclient import LunaWSClient  # pylint: disable-msg=C0415

    serviceName = "Luna Sender"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = LunaWSClient(None, origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaLambda(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Lambda.

    Args:
        origin: Luna Lambda origin
        apiVersion: Luna Lambda api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.luna_lambda.luna_lambda import LambdaApi  # pylint: disable-msg=C0415

    serviceName = "Luna Lambda"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = LambdaApi(origin=origin, api=apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaApi(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna API.

    Args:
        origin: Luna API origin
        apiVersion: Luna API api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.lunavl.httpclient import LunaHttpClient  # pylint: disable-msg=C0415

    serviceName = "Luna API"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = LunaHttpClient(accountId=None, origin=origin, api=apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkConnectionToLunaLicenses(
    origin: str, apiVersion: int, *, postfix: str = "", **kwargs
) -> Union[Awaitable[bool], bool]:
    """
    Function for check connection to Luna Licenses.

    Args:
        origin: Luna Licenses origin
        apiVersion: Luna Licenses api version
        postfix: specified postfix to use in logs

    Returns:
        True, if check was passed successfully, else False.
    """
    from luna3.licenses.licenses import LicensesApi  # pylint: disable-msg=C0415

    serviceName = "Luna Licenses"
    logger = Logger(template=f"Check connection to {serviceName} {postfix}")
    luna3Client = LicensesApi(origin, apiVersion)
    return _checkServiceConnection(logger, luna3Client, serviceName, **kwargs)


def checkDbFunctionExistence(
    dbSetting: "DBSetting",
    saFuncCall: "sqlalchemy.sql.elements.ColumnClause",
    comparator: Callable[[Any], bool],
    *,
    name: str,
    asyncCheck: bool = False,
) -> Union[Awaitable[bool], bool]:
    """
    Try call custom function that should exist in database. Assert expected result.

    Args:
        dbSetting: database setting
        saFuncCall: constructed sqlalchemy function object
        comparator: function for checking `saFuncCall` result
        name: function name for logging
        asyncCheck: whether to return coroutine

    Returns:
        True, if check was passed successfully, else False.
        or
        coroutine
    """
    from sqlalchemy import select  # pylint: disable-msg=C0415

    from .adaptor_engine import UniAdaptorEngine  # pylint: disable-msg=C0415

    logger = Logger(template=f"Check database function '{name}' correctness")
    logger.info("begin")

    async def check():
        try:
            async with UniAdaptorEngine(dbSetting=dbSetting).adaptorContext() as uniAdaptor:
                selectSt = select([saFuncCall])
                dbRes = await uniAdaptor.execute(selectSt)

                if comparator(dbRes[0][0]):
                    logger.info("success")
                    return True

                logger.error(f"function returns incorrect result '{dbRes}'.")
        except TypeError:
            # raised when function got wrong count of args or wrong kwargs
            raise
        except Exception as exc:  # pylint: disable-msg=broad-except
            logger.error(f"check failed, error: {exc}")

        return False

    return syncCallable(check)(asyncCall=asyncCheck)


def checkExistingDocumentsDirectory():
    """
    Function of checking the static directory with document files
    Returns:
        True, if check was passed successfully, else False.
    """
    logger = Logger(template="Check path to the static document directory")
    logger.info("begin")

    staticDirectory = APP_ROOT.joinpath("app", "handlers", "static")

    def getExpectedDocs(servicePath: PosixPath) -> Set[str]:
        """
        Get documentation files used by the service
        Args:
            servicePath: path to the service directory

        Returns:
            tuple with documentation as strings
        """
        filename = servicePath.name.replace("_", "-")
        expectedDocs = {f"{filename}.html"}
        projectDoscFolder = Path().joinpath(servicePath.parent, "docs")
        listUsedDocTypes = [docDir.name for docDir in projectDoscFolder.iterdir() if docDir.is_dir()]
        if "openapi" in listUsedDocTypes:
            expectedDocs.add(f"{filename}.yml")
        if "raml" in listUsedDocTypes:
            expectedDocs.add(f"{filename}.raml")
        return expectedDocs

    if staticDirectory.exists():
        listStaticFiles = [file.name for file in staticDirectory.iterdir()]
        for file in getExpectedDocs(APP_ROOT):
            if file not in listStaticFiles:
                logger.error(f"Document with name: {file}, not found")
                return False
        logger.info("success")
        return True
    logger.error("Static documents directory not found")
    return False


def checkConnectionToLiveness(
    origin: str, timeout: "aiohttp.ClientTimeout", *, postfix: str = "", asyncCheck: bool = False, **kwargs
) -> Union[bool, Awaitable[bool]]:
    """
    Function for check connection to Liveness service

    Args:
        origin: Liveness API origin
        timeout: request timeout
        postfix: specified postfix to use in logs
        asyncCheck: make synchronous or asynchronous check
    Returns:
        True, if check was passed successfully, else False.
    """

    async def check():
        from luna3.common.requests import makeRequest  # pylint: disable-msg=C0415

        logger = Logger(template=f"Check connection to liveness {postfix}")
        logger.info("begin")
        reply = await makeRequest(
            origin + "/api_version",
            method="GET",  # pylint: disable=duplicate-code
            totalTimeout=timeout.total,
            connectTimeout=timeout.connect,
            sockConnectTimeout=timeout.sock_connect,
            sockReadTimeout=timeout.sock_read,
            asyncRequest=True,
            **kwargs,
        )
        if reply.success:
            logger.info("success")
            return True

        logger.error(f"code '{reply.statusCode}' has been returned.")
        return False

    return syncCallable(check)(asyncCall=asyncCheck)


async def serviceCheck(
    checkerName: str,
    client: "luna3.common.base_api.BaseAPI",
    logger: Logger,
    closeSession: Optional[bool] = None,
    totalTimeout: Optional[Real] = None,
    connectTimeout: Optional[Real] = None,
    sockReadTimeout: Optional[Real] = None,
    sockConnectTimeout: Optional[Real] = None,
) -> Health:
    # pylint: disable-msg=W0621
    """
    Service health checker
    Args:
        logger: logger to log
        client: client to check
        closeSession: whether to close session (True - yes, False - no, None - default)
        totalTimeout: total request timeout
        connectTimeout: socket connect timeout or connection wait from pool timeout
        sockReadTimeout: timeout for reading a chunk from socket
        sockConnectTimeout: socket connect timeout
    Returns:
        Health status
    """
    from luna3.common.base_api import BaseHTTPAPI  # pylint: disable-msg=C0415

    kwargs = {
        "lunaRequestId": DEFAULT_REQUEST_ID,
        "raiseError": True,
        "totalTimeout": totalTimeout,
        "asyncRequest": True,
    }
    if closeSession is not None:
        kwargs["closeSession"] = closeSession

    if isinstance(client, BaseHTTPAPI):
        kwargs["connectTimeout"] = connectTimeout
        kwargs["sockReadTimeout"] = sockReadTimeout
        kwargs["sockConnectTimeout"] = sockConnectTimeout
    else:
        # crutch for core services
        kwargs["service"] = checkerName

    try:
        if kwargs["asyncRequest"]:
            healthStatus = await client.testConnection(**kwargs)
        else:
            healthStatus = client.testConnection(**kwargs)
    except LunaApiException as exc:
        logMsg = exc.json["detail"]
        logger.error(logMsg)
        return Health(checkerName, False, Error.HealthCheckError.format(logMsg))
    except Exception as exc:  # pylint: disable-msg=broad-except
        if hasattr(client, "baseUri"):
            logMsg = f"Failed check connection, uri: {client.baseUri}, error: {exc}"
        else:
            logMsg = f"Failed check connection, error: {exc}"
        logger.exception(logMsg)
        return Health(checkerName, False, Error.HealthCheckError.format(str(exc)))

    if not healthStatus:
        errorMsg = "Connection refused"
        logger.error(errorMsg)
        return Health(checkerName, False, Error.HealthCheckError.format(errorMsg))
    return Health(checkerName, True, None)
