"""
Module realizes base proxy handler
"""
from __future__ import annotations

from abc import ABC, abstractmethod
from functools import partial, partialmethod
from typing import Any, Dict, Mapping, NamedTuple, Set, Tuple, Union

from aiohttp import ClientSession, ClientTimeout, TCPConnector
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import DNSResolver, makeRequest
from multidict import CIMultiDictProxy
from sanic.response import HTTPResponse

from ..utils import mixins
from .handlers import BaseHandler


class ProxyRequest(NamedTuple):
    """
    Proxy request
    """

    body: Any  # body for sending to a proxed service
    headers: Union[Dict[str, str], CIMultiDictProxy[str]]  # headers for sending to a proxed service
    query: Dict[str, Union[None, str]]  # query for sending to a proxed service


class SessionPool(mixins.Initializable):
    """
    Descriptor for lazy creation of aiohttp.ClientSession
    Sessions are created per serviceAddress, and every session shares the same connection pool,
    which is managed outside, by application.
    """

    #  A dictionary with mapping between serviceAddress and ClientSession
    _sessions: Dict[str, ClientSession] = {}
    # tcp connector for sessions
    connector = None

    def __get__(self, cls: VLBaseProxyHandler, *args, **kwargs) -> ClientSession:
        """
        Gets or creates appropriate session, based on owners class type.
        Owner class should be sublcass of VLBaseProxyHandler.

        Args:
            cls: The class on which the attribute was accessed

        Returns:
            ClientSession
        """
        if self.connector is None:
            raise RuntimeError("Connector is not initialized!")

        key = (
            f"{cls.serviceAddress.origin}{cls.serviceTimeouts.totalTimeout}"
            f"{cls.serviceTimeouts.connectTimeout}"
            f"{cls.serviceTimeouts.sockConnectTimeout}"
            f"{cls.serviceTimeouts.sockReadTimeout}"
        )
        try:
            return self._sessions[key]
        except KeyError:
            session = ClientSession(
                timeout=ClientTimeout(
                    total=cls.serviceTimeouts.totalTimeout,
                    connect=cls.serviceTimeouts.connectTimeout,
                    sock_connect=cls.serviceTimeouts.sockConnectTimeout,
                    sock_read=cls.serviceTimeouts.sockReadTimeout,
                ),
                connector=self.connector,
                connector_owner=False,
            )
            self._sessions[key] = session
            return session

    @classmethod
    async def initialize(cls):
        """initialize pool (tcp connector)"""
        cls.connector = TCPConnector(resolver=DNSResolver())

    @classmethod
    async def close(cls):
        """close connector and open sessions"""
        if not cls.connector:
            return

        for address, session in list(cls._sessions.items()):
            await session.close()
            del cls._sessions[address]
        await cls.connector.close()


class VLBaseProxyHandler(BaseHandler, ABC):  # pylint: disable-msg=too-many-public-methods
    """
    Base proxy handler for other handlers.
    """

    # list of allowed proxy methods
    allowedMethods: Tuple[str, ...] = ()
    # list of blacklisted headers names for responses
    dirtyResponseHeaders = {"content-length", "content-encoding", "transfer-encoding"}
    # list of blacklisted headers names for requests
    dirtyRequestHeaders = {"content-length", "content-encoding"}
    # session pool
    session = SessionPool()

    @staticmethod
    def cleanHeaders(allHeaders: Mapping[str, str], dirtyHeaders: Set[str]) -> Dict[str, str]:
        """
        Remove dirty headers.

        Args:
            allHeaders: headers to filter
            dirtyHeaders: Set of header names

        Returns:
            filtered headers (new instance)
        """
        filteredHeaders = {key: value for key, value in allHeaders.items() if key.lower() not in dirtyHeaders}
        return filteredHeaders

    cleanRequestHeaders: partial = partialmethod(cleanHeaders, dirtyHeaders=dirtyRequestHeaders)
    cleanResponseHeaders: partial = partialmethod(cleanHeaders, dirtyHeaders=dirtyResponseHeaders)
    filterAllowedHeaders = cleanRequestHeaders  # COMPAT

    @property
    @abstractmethod
    def serviceAddress(self) -> "ServiceAddressSettings":
        """
        Get a proxed service address
        Returns:
            a proxed service address
        """

    @property
    @abstractmethod
    def serviceTimeouts(self) -> "ServiceTimeoutsSettings":
        """
        Get a proxed service timeouts
        Returns:
            a proxed service timeouts
        """

    @property
    def serviceUrl(self) -> str:
        """Service url"""
        return f"{self.serviceAddress.origin}/{self.serviceAddress.apiVersion}"

    def prepareRequestDefault(self) -> ProxyRequest:
        """
        Prepare proxy request by default
        Returns:
            default proxy request
        """
        return ProxyRequest(self.request.body, self.prepareHeaders(), self.prepareQuery())

    async def prepareRequestPost(self):
        """
        Prepare proxy request for method `post`
        Returns:
            proxy request
        """
        return self.prepareRequestDefault()

    async def prepareRequestPatch(self):
        """
        Prepare proxy request for method `patch`
        Returns:
            proxy request
        """
        return self.prepareRequestDefault()

    async def prepareRequestDelete(self):
        """
        Prepare proxy request for method `delete`
        Returns:
            proxy request
        """
        return self.prepareRequestDefault()

    async def prepareRequestPut(self):
        """
        Prepare proxy request for method `put`
        Returns:
            proxy request
        """
        return self.prepareRequestDefault()

    async def prepareProxyRequest(self) -> ProxyRequest:
        """
        Prepare proxy request
        Returns:
            proxy request
        """
        if self.request.method in ("GET", "OPTIONS", "HEAD"):
            return ProxyRequest(b"", self.prepareHeaders(), self.prepareQuery())
        if self.request.method == "POST":
            return await self.prepareRequestPost()
        if self.request.method == "PUT":
            return await self.prepareRequestPut()
        if self.request.method == "DELETE":
            return await self.prepareRequestDelete()
        if self.request.method == "PATCH":
            return await self.prepareRequestPatch()
        raise RuntimeError(f"bad request method {self.request.method}")

    def success(self, *args, extraHeaders=None, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0221
        """Clean headers (if passed)"""
        if extraHeaders is not None:
            extraHeaders = self.cleanResponseHeaders(extraHeaders)
        return super().success(*args, extraHeaders=extraHeaders, **kwargs)

    async def postProcessingDefault(self, response: LunaResponse) -> HTTPResponse:
        """
        Default post processing response from the service
        Args:
            response: response

        Returns:
            response in api format
        """
        headers = dict(response.headers)
        if "Location" in response.headers:
            headers["Location"] = response.headers["Location"].replace(
                f"/{self.serviceAddress.apiVersion}/", f"/{self.app.ctx.apiVersion}/", 1
            )
        return self.success(response.statusCode, body=response.body, extraHeaders=headers)

    async def postProcessingFailedRequest(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing failed response from the service
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingPost(self, response: LunaResponse) -> HTTPResponse:
        """
        Default post processing response from the service
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingGet(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response from the service for method `get`
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingPatch(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response from the service for method `patch`
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingDelete(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response from the service for method `delete`
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingPut(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response from the service for method `put`
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingOptions(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response from the service for method `post`
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessingHead(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response from the service for method `head`
        Args:
            response: response

        Returns:
            response in api format
        """
        return await self.postProcessingDefault(response)

    async def postProcessing(self, response: LunaResponse) -> HTTPResponse:
        """
        Post processing response
        Args:
            response: response

        Returns:
            response in api format
        """
        # pylint: disable-msg=too-many-return-statements

        if self.request.method == "GET":
            return await self.postProcessingGet(response)
        if self.request.method == "OPTIONS":
            return await self.postProcessingOptions(response)
        if self.request.method == "HEAD":
            return await self.postProcessingHead(response)
        if self.request.method == "POST":
            return await self.postProcessingPost(response)
        if self.request.method == "PUT":
            return await self.postProcessingPut(response)
        if self.request.method == "DELETE":
            return await self.postProcessingDelete(response)
        if self.request.method == "PATCH":
            return await self.postProcessingPatch(response)
        raise RuntimeError(f"bad request method {self.request.method}")

    def prepareQuery(self) -> Dict[str, str]:
        """
        Prepare queries for a proxy request
        Returns:
            dict with queries
        """
        queryParams = dict(self.request.args)
        return queryParams

    def prepareHeaders(self) -> Dict[str, str]:
        """
        Prepare headers for a proxy request
        Returns:
            headers with `Luna-Request-Id` and other input headers
        """
        headers = self.request.headers.copy()
        headers["Luna-Request-Id"] = self.requestId
        # aiohttp set 'deflate, gzip'  by default. Add `identity`, service choose a compress algorithm.
        # aiohttp decompress response body automaticly, input Accept-Encoding processing will processing in middleware
        headers["Accept-Encoding"] = "identity, deflate, gzip"
        return headers

    def prepareUrl(self) -> str:
        """
        Prepare url to a service
        Returns:
            same url with correct api version
        """
        uriWithoutApiVersion = self.request.server_path.replace(f"/{self.app.ctx.apiVersion}", "", 1)
        return f"{self.serviceUrl}{uriWithoutApiVersion}"

    async def makeRequestToService(self) -> HTTPResponse:
        """
        Make request to custom service
        Returns:
            response
        """
        if self.request.method not in self.allowedMethods:
            return self.setMethodNotAllowed()
        url = self.prepareUrl()
        preparedRequest = await self.prepareProxyRequest()

        kwargs = {
            "url": url,
            "method": self.request.method,
            "queryParams": preparedRequest.query,
            "headers": self.cleanRequestHeaders(preparedRequest.headers),
            "asyncRequest": True,
            "session": self.session,
        }

        reply = await makeRequest(body=preparedRequest.body, **kwargs)

        if reply.success:
            return await self.postProcessing(reply)
        return await self.postProcessingFailedRequest(reply)

    async def options(self, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0613
        """
        Default 'options' method
        """
        allowHeaders = {"Allow": ", ".join(self.getAllowedMethods())}
        return self.success(extraHeaders=allowHeaders)

    async def post(self, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0613
        """
        Default 'post' method
        Returns:
            proxy response
        """
        return await self.makeRequestToService()

    async def put(self, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0613
        """
        Default 'put' method
        Returns:
            proxy response
        """
        return await self.makeRequestToService()

    async def get(self, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0613
        """
        Default 'get' method
        Returns:
            proxy response
        """
        return await self.makeRequestToService()

    async def delete(self, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0613
        """
        Default 'delete' method
        Returns:
            proxy response
        """
        return await self.makeRequestToService()

    async def patch(self, **kwargs) -> HTTPResponse:  # pylint: disable-msg=W0613
        """
        Default 'patch' method
        Returns:
            proxy response
        """
        return await self.makeRequestToService()

    async def head(self, **kwargs):  # pylint: disable-msg=W0613
        """
        Default 'head' method
        Returns:
            proxy response
        """
        return await self.makeRequestToService()
