"""Health Handler

Module realizes healthcheck handler.
"""
import asyncio
from itertools import chain
from time import perf_counter

from sanic.response import HTTPResponse, redirect

from ..utils.healthcheck import Health, performCheck
from ..web import response
from ..web.handlers import BaseHandler
from .query_getters import int01Getter


class HealthCheckHandler(BaseHandler):  # pylint: disable-msg=W0223
    """
    Handler for health check
    """

    async def checkHealth(self, includeLunaServices: bool = False) -> list[Health]:
        """
        Run checks simultaneously, return list of checks results.

        Args:
            includeLunaServices: A bool, whether to return checks for luna services.
        """
        waiters = [performCheck(name, check) for name, check in self.app.ctx.getRuntimeChecks(includeLunaServices)]
        results = await asyncio.gather(*waiters)
        return sorted(list(chain(*results)), key=lambda check: check.status)

    async def get(self) -> HTTPResponse:
        """
        Check health of service, see `spec`_

        .. _spec:
            _static/api.html#operation/healthcheck

        Resource is reached by address '/healthcheck'

        """
        start = perf_counter()

        includeLunaServices = bool(self.getQueryParam("include_luna_services", int01Getter, default=0))
        results = await self.checkHealth(includeLunaServices=includeLunaServices)
        if not all(health.status for health in results):
            outputJson = {"errors": [health.asDict() for health in results]}
            return response.json(status=502, body=outputJson, headers=self.respHeaders)

        outputJson = {"execution_time": perf_counter() - start}
        return response.json(status=200, body=outputJson, headers=self.respHeaders)


class HealthCheckRedirectHandler(BaseHandler):
    """Redirect handler."""

    async def get(self) -> HTTPResponse:
        """Redirect request to the HealthCheckHandler"""
        url = self.app.url_for("HealthCheckHandler")
        if self.request.query_string:
            url += "?" + self.request.query_string
        return redirect(url, status=308)
